;;;-*- Mode: Lisp; Package: CCL -*-
;;;
;;;   Copyright (C) 1994-2001 Digitool, Inc
;;;   This file is part of Opensourced MCL.
;;;
;;;   Opensourced MCL is free software; you can redistribute it and/or
;;;   modify it under the terms of the GNU Lesser General Public
;;;   License as published by the Free Software Foundation; either
;;;   version 2.1 of the License, or (at your option) any later version.
;;;
;;;   Opensourced MCL is distributed in the hope that it will be useful,
;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;   Lesser General Public License for more details.
;;;
;;;   You should have received a copy of the GNU Lesser General Public
;;;   License along with this library; if not, write to the Free Software
;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;;;


; l0-cfm-support.lisp



; offset is a fixnum, one of the arch::kernel-import-xxx.
; Returns that kernel import, a fixnum.
#+ppc-target
(defppclapfunction %kernel-import ((offset arg_z))
  (ref-global imm0 kernel-imports)
  (unbox-fixnum imm1 arg_z)
  (lwzx arg_z imm0 imm1)
  (blr))

#+sparc-target
(defsparclapfunction %kernel-import ((%offset %arg_z))
  (ref-global %imm0 kernel-imports)
  (unbox-fixnum %offset %imm1)
  (retl)
    (ld (%imm0 %imm1) %arg_z))

#+ppc-target
(defppclapfunction %get-unboxed-ptr ((macptr arg_z))
  (macptr-ptr imm0 arg_z)
  (lwz arg_z 0 imm0)
  (blr))

#+sparc-target
(defsparclapfunction %get-unboxed-ptr ((%macptr %arg_z))
  (macptr-ptr %macptr %imm0)
  (retl)
   (ld (%imm0) %arg_z))


; Bootstrapping. Real version is in l1-aprims.
; Called by expansion of with-pstrs

(defun byte-length (string &optional script start end)
    (declare (ignore script))
    (when (or start end)
      (error "Don't support start or end args yet"))
    (if (base-string-p string)
      (length string)
      (error "Don't support non base-string yet.")))



(def-accessor-macros %svref
  nil                                 ; 'external-entry-point
  eep.address
  eep.name
  eep.container)

(defun %cons-external-entry-point (name &optional container)
  (%istruct 'external-entry-point nil name container))

(defun external-entry-point-p (x)
  (istruct-typep x 'external-entry-point))

(defvar *rtld-next*)
(defvar *rtld-default*)
(setq *rtld-next* (%int-to-ptr #xFFFFFFFF)
      *rtld-default* (%int-to-ptr 0))


(defvar *eeps* nil)

(defun eeps ()
  (or *eeps*
      (setq *eeps* (make-hash-table :test #'equal))))

(defun resolve-container (c)
  (if c
    (error "Don't handle external containers yet.")
    *rtld-default*))

(defun resolve-eep (e)
  (or (eep.address e)
      (let* ((name (eep.name e))
             (handle (resolve-container (eep.container e))))
        (with-cstrs ((n name))
          (let* ((addr (ff-call (%kernel-import arch::kernel-import-dlsym) :address handle :address n :signed-fullword)))
            (declare (integer addr))
            (if (zerop addr)
              (error "Can't resolve external-entrypoint ~s" name)
              (setf (eep.address e) (ash addr -2))))))))

          
(defun load-eep (name)
  (let* ((eep (or (gethash name (eeps)) (setf (gethash name *eeps*) (%cons-external-entry-point name)))))
    (resolve-eep eep)
    eep))

#+ppc-target
(defppclapfunction %revive-macptr ((p arg_z))
  (li imm0 arch:subtag-macptr)
  (stb imm0 arch::misc-subtag-offset p)
  (blr))

#+sparc-target
(defsparclapfunction %revive-macptr ((p %arg_z))
  (mov arch::subtag-macptr %imm0)
  (retl)
  (stb %imm0 (p arch::misc-subtag-offset)))

(defun refresh-external-entrypoints ()
  (%revive-macptr *rtld-next*)
  (%revive-macptr *rtld-default*)
  (when *eeps*
    (without-interrupts 
     (maphash #'(lambda (k v) 
                  (declare (ignore k)) 
                  (setf (eep.address v) nil) 
                  (resolve-eep v))
              *eeps*))))




